@@ -25,7 +25,6 @@
 #include <gaim.h>
 #include <gaim/version.h>
 #include <gaim/debug.h>
-#include <gaim/away.h>
 #include <gaim/gtkblist.h>
 #include <gaim/gtkplugin.h>
 #include <gaim/gtkutils.h>
@@ -59,9 +58,6 @@
 GaimPlugin *handle = NULL;
 #ifdef G_OS_WIN32
 __declspec(dllimport) GSList *unread_message_queue;
-__declspec(dllimport) GSList *away_messages;
-__declspec(dllimport) struct away_message *awaymessage;
-__declspec(dllimport) GSList *message_queue;
 #endif
 
 static gchar* gen_keystr(HotkeyEntry* key, GtkWidget* widget);
@@ -112,13 +108,49 @@ gen_keystr(HotkeyEntry* key, GtkWidget* widget)
     return str;
 }
 
+static GList *
+get_pending_list(guint max)
+{
+	const char *im = gaim_prefs_get_string("/plugins/gtk/docklet/blink_im");
+	const char *chat = gaim_prefs_get_string("/plugins/gtk/docklet/blink_chat");
+	GList *l_im = NULL;
+	GList *l_chat = NULL;
+
+	if (im != NULL && strcmp(im, "always") == 0) {
+		l_im = gaim_gtk_conversations_find_unseen_list(GAIM_CONV_TYPE_IM,
+				GAIM_UNSEEN_TEXT,
+				FALSE, max);
+	} else if (im != NULL && strcmp(im, "hidden") == 0) {
+		l_im = gaim_gtk_conversations_find_unseen_list(GAIM_CONV_TYPE_IM,
+				GAIM_UNSEEN_TEXT,
+				TRUE, max);
+	}
+
+	if (chat != NULL && strcmp(chat, "always") == 0) {
+		l_chat = gaim_gtk_conversations_find_unseen_list(GAIM_CONV_TYPE_CHAT,
+				GAIM_UNSEEN_TEXT,
+				FALSE, max);
+	} else if (chat != NULL && strcmp(chat, "nick") == 0) {
+		l_chat = gaim_gtk_conversations_find_unseen_list(GAIM_CONV_TYPE_CHAT,
+				GAIM_UNSEEN_NICK,
+				FALSE, max);
+	}
+
+	if (l_im != NULL && l_chat != NULL)
+		return g_list_concat(l_im, l_chat);
+	else if (l_im != NULL)
+		return l_im;
+	else
+		return l_chat;
+}
+
 static gboolean
 event_filter(gpointer event_data)
 {
     switch (real_event_filter(event_data, hotkeys, num_hotkeys))
     {
         case GAIM_HOTKEY_TOGGLE_BLIST:
-            gaim_gtk_blist_docklet_toggle();
+            gaim_gtk_blist_toggle_visibility();
 #ifdef GDK_WINDOWING_X11
 	    {
 		GaimGtkBuddyList *gtkblist;
@@ -132,52 +164,26 @@ event_filter(gpointer event_data)
 #endif
             break;
         case GAIM_HOTKEY_READ_MSG:
-            if (unread_message_queue)
+	{
+	    GList *l;
+
+            if ((l = get_pending_list(1)))
 	    {
-#ifdef GDK_WINDOWING_X11
-		struct temp_storage {
-		    GaimAccount *account;
-		    gchar *name;
-		} *ts;
-		GSList *node, *my_queue = NULL;
-		struct queued_message *qm;
-		GaimConversation *cnv, *prev_cnv;
-
-		for (node = unread_message_queue; node; node = node->next)
-		{
-		    ts = g_new(struct temp_storage, 1);
-		    qm = node->data;
-		    ts->account = qm->account;
-		    ts->name = g_strdup(qm->name);
-		    my_queue = g_slist_prepend(my_queue, ts);
-		}
-		my_queue = g_slist_reverse(my_queue);
-#endif
-
-                purge_away_queue(&unread_message_queue);
+		gaim_gtkconv_present_conversation((GaimConversation *)l->data);
 
 #ifdef GDK_WINDOWING_X11
-		prev_cnv = NULL;
-		for (node = my_queue; node; node = node->next)
 		{
-		    ts = (struct temp_storage *) node->data;
-		    cnv = gaim_find_conversation_with_account(ts->name, ts->account);
-		    if (cnv && cnv != prev_cnv)
-		    {
-			GtkWindow *gtkwindow;
-			
-			gtkwindow = GTK_WINDOW(GAIM_GTK_WINDOW(gaim_conversation_get_window(cnv))->window);
-			gtk_window_present(gtkwindow);
-			hacky_active_window(GTK_WIDGET(gtkwindow));
-		    }
-		    prev_cnv = cnv;
-		    g_free(ts->name);
-		    g_free(ts);
+		    GtkWindow *gtkwindow;
+
+		    gtkwindow = GTK_WINDOW(gaim_gtkconv_get_window(GAIM_GTK_CONVERSATION((GaimConversation *)l->data))->window);
+		    /*gtk_window_present(gtkwindow);*/
+		    hacky_active_window(GTK_WIDGET(gtkwindow));
 		}
-		g_slist_free(my_queue);
 #endif
+		g_list_free(l);
 	    }
             break;
+	}
 	case GAIM_HOTKEY_SHOW_PREF:
 	    gaim_gtk_prefs_show();
             break;
